@@ -37,6 +37,16 @@
 #include "itclMigrate2TclCore.h"
 #include "itclTclIntStubsFcn.h"
 
+/*
+ * Utility macros: STRINGIFY takes an argument and wraps it in "" (double
+ * quotation marks).
+ */
+
+#ifndef STRINGIFY
+#  define STRINGIFY(x) STRINGIFY1(x)
+#  define STRINGIFY1(x) #x
+#endif
+
 /*
  * Since the Tcl/Tk distribution doesn't perform any asserts,
  * dynamic loading can fail to find the __assert function.
@@ -456,6 +466,8 @@ typedef struct ItclMemberCode {
 #define ITCL_OPTION_READONLY   0x2000 /* non-zero => readonly */
 #define ITCL_VARIABLE          0x4000 /* non-zero => normal variable */
 #define ITCL_TYPE_VARIABLE     0x8000 /* non-zero => typevariable */
+#define ITCL_OPTION_INITTED    0x10000 /* non-zero => option has been initialized */
+#define ITCL_OPTION_COMP_VAR   0x20000 /* variable to collect option components of extendedclass  */
 
 /*
  *  Instance components.
@@ -683,9 +695,9 @@ MODULE_SCOPE int ItclCheckCallProc(ClientData clientData, Tcl_Interp *interp,
 MODULE_SCOPE ItclFoundation *ItclGetFoundation(Tcl_Interp *interp);
 MODULE_SCOPE Tcl_ObjCmdProc ItclClassCommandDispatcher;
 MODULE_SCOPE Tcl_Command Itcl_CmdAliasProc(Tcl_Interp *interp,
-        Tcl_Namespace *nsPtr, CONST char *cmdName, ClientData clientData);
+        Tcl_Namespace *nsPtr, const char *cmdName, ClientData clientData);
 MODULE_SCOPE Tcl_Var Itcl_VarAliasProc(Tcl_Interp *interp,
-        Tcl_Namespace *nsPtr, CONST char *VarName, ClientData clientData);
+        Tcl_Namespace *nsPtr, const char *VarName, ClientData clientData);
 MODULE_SCOPE int ItclIsClass(Tcl_Interp *interp, Tcl_Command cmd);
 MODULE_SCOPE int ItclCheckCallMethod(ClientData clientData, Tcl_Interp *interp,
         Tcl_ObjectContext contextPtr, Tcl_CallFrame *framePtr, int *isFinished);
@@ -704,8 +716,8 @@ MODULE_SCOPE int ItclCreateArgList(Tcl_Interp *interp, const char *str,
 	const char *commandName);
 MODULE_SCOPE int ItclObjectCmd(ClientData clientData, Tcl_Interp *interp,
         Tcl_Object oPtr, Tcl_Class clsPtr, int objc, Tcl_Obj *const *objv);
-MODULE_SCOPE int ItclCreateObject (Tcl_Interp *interp, CONST char* name,
-        ItclClass *iclsPtr, int objc, Tcl_Obj *CONST objv[]);
+MODULE_SCOPE int ItclCreateObject (Tcl_Interp *interp, const char* name,
+        ItclClass *iclsPtr, int objc, Tcl_Obj *const objv[]);
 MODULE_SCOPE void ItclDeleteObjectVariablesNamespace(Tcl_Interp *interp,
         ItclObject *ioPtr);
 MODULE_SCOPE void ItclDeleteClassVariablesNamespace(Tcl_Interp *interp,
@@ -715,24 +727,22 @@ MODULE_SCOPE char * ItclTraceUnsetVar(ClientData clientData, Tcl_Interp *interp,
 	const char *name1, const char *name2, int flags);
 
 struct Tcl_ResolvedVarInfo;
-MODULE_SCOPE int Itcl_ClassCmdResolver(Tcl_Interp *interp, CONST char* name,
+MODULE_SCOPE int Itcl_ClassCmdResolver(Tcl_Interp *interp, const char* name,
 	Tcl_Namespace *nsPtr, int flags, Tcl_Command *rPtr);
-MODULE_SCOPE int Itcl_ClassVarResolver(Tcl_Interp *interp, CONST char* name,
+MODULE_SCOPE int Itcl_ClassVarResolver(Tcl_Interp *interp, const char* name,
         Tcl_Namespace *nsPtr, int flags, Tcl_Var *rPtr);
 MODULE_SCOPE int Itcl_ClassCompiledVarResolver(Tcl_Interp *interp,
-        CONST char* name, int length, Tcl_Namespace *nsPtr,
+        const char* name, int length, Tcl_Namespace *nsPtr,
         struct Tcl_ResolvedVarInfo **rPtr);
-MODULE_SCOPE int Itcl_ClassCmdResolver2(Tcl_Interp *interp, CONST char* name,
+MODULE_SCOPE int Itcl_ClassCmdResolver2(Tcl_Interp *interp, const char* name,
 	Tcl_Namespace *nsPtr, int flags, Tcl_Command *rPtr);
-MODULE_SCOPE int Itcl_ClassVarResolver2(Tcl_Interp *interp, CONST char* name,
+MODULE_SCOPE int Itcl_ClassVarResolver2(Tcl_Interp *interp, const char* name,
         Tcl_Namespace *nsPtr, int flags, Tcl_Var *rPtr);
 MODULE_SCOPE int Itcl_ClassCompiledVarResolver2(Tcl_Interp *interp,
-        CONST char* name, int length, Tcl_Namespace *nsPtr,
+        const char* name, int length, Tcl_Namespace *nsPtr,
         struct Tcl_ResolvedVarInfo **rPtr);
 MODULE_SCOPE int ItclSetParserResolver(Tcl_Namespace *nsPtr);
 MODULE_SCOPE void ItclProcErrorProc(Tcl_Interp *interp, Tcl_Obj *procNameObj);
-MODULE_SCOPE int ItclClassBaseCmd(ClientData clientData, Tcl_Interp *interp,
-	int flags, int objc, Tcl_Obj *CONST objv[], ItclClass **iclsPtrPtr);
 MODULE_SCOPE int Itcl_CreateOption (Tcl_Interp *interp, ItclClass *iclsPtr,
 	ItclOption *ioptPtr);
 MODULE_SCOPE int Itcl_CreateMethodVariable (Tcl_Interp *interp,
@@ -746,30 +756,24 @@ MODULE_SCOPE const char* ItclGetInstanceVar(Tcl_Interp *interp,
 MODULE_SCOPE const char* ItclGetCommonInstanceVar(Tcl_Interp *interp,
         const char *name, const char *name2, ItclObject *contextIoPtr,
 	ItclClass *contextIclsPtr);
-MODULE_SCOPE const char* ItclSetInstanceVar(Tcl_Interp *interp,
-        const char *name, const char *name2, const char *value,
-	ItclObject *contextIoPtr, ItclClass *contextIclsPtr);
-MODULE_SCOPE Tcl_Obj * ItclCapitalize(const char *str);
 MODULE_SCOPE int ItclCreateMethod(Tcl_Interp* interp, ItclClass *iclsPtr,
 	Tcl_Obj *namePtr, const char* arglist, const char* body,
         ItclMemberFunc **imPtrPtr);
-MODULE_SCOPE int ItclCreateComponent(Tcl_Interp *interp, ItclClass *iclsPtr,
-        Tcl_Obj *componentPtr, int type, ItclComponent **icPtrPtr);
 MODULE_SCOPE int Itcl_WidgetParseInit(Tcl_Interp *interp,
         ItclObjectInfo *infoPtr);
 MODULE_SCOPE void ItclDeleteObjectMetadata(ClientData clientData);
 MODULE_SCOPE void ItclDeleteClassMetadata(ClientData clientData);
 MODULE_SCOPE void ItclDeleteArgList(ItclArgList *arglistPtr);
 MODULE_SCOPE int Itcl_ClassOptionCmd(ClientData clientData, Tcl_Interp *interp,
-        int objc, Tcl_Obj *CONST objv[]);
+        int objc, Tcl_Obj *const objv[]);
 MODULE_SCOPE int DelegatedOptionsInstall(Tcl_Interp *interp,
         ItclClass *iclsPtr);
 MODULE_SCOPE int Itcl_HandleDelegateOptionCmd(Tcl_Interp *interp,
         ItclObject *ioPtr, ItclClass *iclsPtr, ItclDelegatedOption **idoPtrPtr,
-        int objc, Tcl_Obj *CONST objv[]);
+        int objc, Tcl_Obj *const objv[]);
 MODULE_SCOPE int Itcl_HandleDelegateMethodCmd(Tcl_Interp *interp,
         ItclObject *ioPtr, ItclClass *iclsPtr,
-	ItclDelegatedFunction **idmPtrPtr, int objc, Tcl_Obj *CONST objv[]);
+	ItclDelegatedFunction **idmPtrPtr, int objc, Tcl_Obj *const objv[]);
 MODULE_SCOPE int DelegateFunction(Tcl_Interp *interp, ItclObject *ioPtr,
         ItclClass *iclsPtr, Tcl_Obj *componentNamePtr,
         ItclDelegatedFunction *idmPtr);
@@ -780,7 +784,7 @@ MODULE_SCOPE ItclOption* ItclNewOption(Tcl_Interp *interp, ItclObject *ioPtr,
         ItclClass *iclsPtr, Tcl_Obj *namePtr, const char *resourceName,
         const char *className, char *init, ItclMemberCode *mCodePtr);
 MODULE_SCOPE int ItclParseOption(ItclObjectInfo *infoPtr, Tcl_Interp *interp,
-        int objc, Tcl_Obj *CONST objv[], ItclClass *iclsPtr,
+        int objc, Tcl_Obj *const objv[], ItclClass *iclsPtr,
 	ItclObject *ioPtr, ItclOption **ioptPtrPtr);
 MODULE_SCOPE void ItclDestroyClassNamesp(ClientData cdata);
 MODULE_SCOPE int ExpandDelegateAs(Tcl_Interp *interp, ItclObject *ioPtr,
